package pdftoppm

import (
	"bytes"
	"context"
	"fmt"
	"os/exec"

	"github.com/rs/zerolog"

	"orus.io/orus-io/rednerd/rendering"
)

func NewEngine() (*Engine, error) {
	e := Engine{}

	return &e, nil
}

var engineInfo = rendering.EngineInfo{
	Type:        "pdftoppm",
	Description: "Use Poppler 'pdftoppm' utility to convert pdfs first page to images",
	Transitions: []rendering.EngineTransitionInfo{
		{FromType: "application/pdf", ToType: "image/png", Priority: 1},
		{FromType: "application/pdf", ToType: "image/jpeg", Priority: 1},
		{FromType: "application/pdf", ToType: "image/tiff", Priority: 1},
	},
	Metadata: []rendering.EngineMetadataInfo{
		{Name: "PDF-page", Default: "1", Description: "Tells which page of the pdf should be converted"},
	},
}

type Engine struct{}

func (e *Engine) Info() rendering.EngineInfo {
	return engineInfo
}

func (e *Engine) Render(
	ctx context.Context, w rendering.DocumentWriter, r rendering.DocumentReader, toType string,
) error {
	args := make([]string, 0, 3)

	switch toType {
	case "image/png":
		args = append(args, "-png")
	case "image/jpeg":
		args = append(args, "-jpeg")
	case "image/tiff":
		args = append(args, "-tiff")
	}

	m := w.Metadata()

	pdfpage := m["PDF-page"]
	if pdfpage == "" {
		pdfpage = "1"
	}

	args = append(args, "-f", pdfpage)

	cmd := exec.Command("/usr/bin/pdftoppm", args...)

	var stderr, stdout bytes.Buffer

	cmd.Stdin = r
	cmd.Stdout = &stdout
	cmd.Stderr = &stderr

	if err := cmd.Run(); err != nil {
		return fmt.Errorf("%w: %s - %s", err, stdout.String(), stderr.String())
	}

	if _, err := w.Write(stdout.Bytes()); err != nil {
		return err
	}

	return nil
}

func init() {
	rendering.RegisterRenderingEngineSetup(
		"pdftoppm",
		func() interface{} {
			return nil
		},
		func(optionGroup interface{}, log zerolog.Logger) (rendering.Engine, error) {
			return NewEngine()
		},
	)
}
