package models

import (
	"database/sql"
	"math"
	"time"

	"github.com/jackc/pgtype"
	"orus.io/orus-io/go-orusapi/database"
)

// dbtable:"render_logs" dbpkey:"id"
type RenderLog struct {
	ID                      int64          `db:"id"`
	RenderedAt              time.Time      `db:"rendered_at"`
	Duration                int            `db:"duration"`
	FromType                string         `db:"from_type"`
	ToType                  string         `db:"to_type"`
	Username                string         `db:"username"`
	PipelineSteps           pgtype.JSONB   `db:"pipeline_steps"`
	TemplateID              sql.NullInt64  `db:"template_id"`
	TemplateInternalVersion sql.NullInt64  `db:"template_internal_version"`
	TemplateLanguage        sql.NullString `db:"template_language"`
	DebugQuery              pgtype.JSONB   `db:"debug_query"`
}

func (m *RenderLog) SetDuration(d time.Duration) {
	m.Duration = int(d / time.Millisecond)
}

func (m *RenderLog) SetDebugQuery(r *RenderRequest) {
	_ = m.DebugQuery.Set(r)
}

func (m *RenderLog) SetTemplate(templateID, templateInternalVersion uint64, templateLanguage string) {
	if templateID != 0 {
		if templateID > math.MaxInt64 {
			panic("templateID integer overflow")
		}

		m.TemplateID.Int64 = int64(templateID)
		m.TemplateID.Valid = true
	}

	if templateInternalVersion != 0 {
		if templateInternalVersion > math.MaxInt64 {
			panic("templateInternalVersion integer overflow")
		}

		m.TemplateInternalVersion.Int64 = int64(templateInternalVersion)
		m.TemplateInternalVersion.Valid = true
	}

	if templateLanguage != "" {
		m.TemplateLanguage.String = templateLanguage
		m.TemplateLanguage.Valid = true
	}
}

func NewRenderLog(
	when time.Time,
	fromType, toType string,
	username string,
) *RenderLog {
	rlog := RenderLog{
		RenderedAt: when,
		FromType:   fromType,
		ToType:     toType,
		Username:   username,
	}

	_ = rlog.PipelineSteps.Set(map[string]string{})
	_ = rlog.DebugQuery.Set("null")

	return &rlog
}

func LogRender(
	db database.SQLHelper,
	when time.Time, duration time.Duration,
	fromType, toType string,
	username string,
	templateID, templateInternalVersion uint64,
	templateLanguage string,
) error {
	rlog := NewRenderLog(when, fromType, toType, username)
	rlog.SetDuration(duration)
	rlog.SetTemplate(templateID, templateInternalVersion, templateLanguage)

	if _, err := db.InsertNoPKey(rlog); err != nil {
		return err
	}

	return nil
}
