package rendering

import (
	"encoding/base64"
	"errors"
	"io"
	"strings"
	"sync"

	"orus.io/orus-io/rednerd/models"
)

type DocumentWriter interface {
	io.WriteCloser

	SetType(mimetype string)
	Metadata() models.Metadata
	CloseMetadata() error
	RenderErrors(errors ...models.RenderError)
}

type DocumentReader interface {
	io.ReadCloser

	Metadata() (string, models.Metadata, error)
	RenderErrors() models.RenderErrors
}

type documentPipe struct {
	bodyWriter *io.PipeWriter
	bodyReader *io.PipeReader

	mimetype       string
	metadata       models.Metadata
	metadataClosed bool

	renderErrors models.RenderErrors

	err error

	l    sync.Mutex
	cond *sync.Cond
}

func (p *documentPipe) CloseMetadata() error {
	p.l.Lock()
	defer p.l.Unlock()

	if p.metadataClosed {
		return nil
	}

	if p.err != nil {
		return p.err
	}

	p.metadataClosed = true
	p.cond.Signal()

	return nil
}

func (p *documentPipe) Close() error {
	p.l.Lock()
	defer p.l.Unlock()

	if p.err != nil {
		return p.err
	}

	if err := p.bodyWriter.Close(); err != nil {
		p.err = err
		p.cond.Signal()

		return err
	}

	p.err = io.ErrClosedPipe
	p.cond.Signal()

	return nil
}

func (p *documentPipe) WaitForMetadata() error {
	p.l.Lock()
	for p.err == nil && !p.metadataClosed {
		p.cond.Wait()
	}

	err := p.err
	if errors.Is(err, io.ErrClosedPipe) {
		err = nil
	}
	p.l.Unlock()

	return err
}

func (p *documentPipe) WaitForClose() error {
	p.l.Lock()
	for p.err == nil {
		p.cond.Wait()
	}

	err := p.err
	p.l.Unlock()

	return err
}

type DocumentPipeReader struct {
	p *documentPipe
}

func (r *DocumentPipeReader) Metadata() (string, models.Metadata, error) {
	if err := r.p.WaitForMetadata(); err != nil {
		return "", nil, err
	}

	return r.p.mimetype, r.p.metadata, nil
}

func (r *DocumentPipeReader) Read(data []byte) (int, error) {
	if err := r.p.WaitForMetadata(); err != nil {
		return 0, err
	}

	return r.p.bodyReader.Read(data)
}

func (r *DocumentPipeReader) RenderErrors() models.RenderErrors {
	_ = r.p.WaitForClose()

	return r.p.renderErrors
}

func (r *DocumentPipeReader) Close() error {
	return r.p.Close()
}

type DocumentPipeWriter struct {
	p *documentPipe
}

func (w *DocumentPipeWriter) SetType(mimetype string) {
	w.p.mimetype = mimetype
}

func (w *DocumentPipeWriter) Metadata() models.Metadata {
	return w.p.metadata
}

func (w *DocumentPipeWriter) CloseMetadata() error {
	return w.p.CloseMetadata()
}

func (w *DocumentPipeWriter) Write(data []byte) (int, error) {
	if err := w.p.CloseMetadata(); err != nil {
		return 0, err
	}

	return w.p.bodyWriter.Write(data)
}

func (w *DocumentPipeWriter) RenderErrors(errors ...models.RenderError) {
	w.p.renderErrors = append(w.p.renderErrors, errors...)
}

func (w *DocumentPipeWriter) Close() error {
	return w.p.Close()
}

func DocumentPipe() (*DocumentPipeWriter, *DocumentPipeReader) {
	r, w := io.Pipe()
	pipe := documentPipe{
		bodyReader: r,
		bodyWriter: w,
		metadata:   make(models.Metadata),
	}
	pipe.cond = sync.NewCond(&pipe.l)

	return &DocumentPipeWriter{&pipe}, &DocumentPipeReader{&pipe}
}

type documentReader struct {
	document *models.Document
	r        io.Reader
}

func (r *documentReader) Metadata() (string, models.Metadata, error) {
	return r.document.Type, r.document.Metadata, nil
}

func (r *documentReader) RenderErrors() models.RenderErrors {
	return r.document.RenderErrors
}

func (r *documentReader) Read(p []byte) (int, error) {
	return r.r.Read(p)
}

func (r *documentReader) Close() error {
	return nil
}

func NewDocumentReader(document *models.Document) DocumentReader {
	var r io.Reader = strings.NewReader(document.Body)
	if document.BodyFormat == "base64" {
		r = base64.NewDecoder(base64.StdEncoding, r)
	}

	return &documentReader{
		document: document,
		r:        r,
	}
}

type documentWriter struct {
	document *models.Document
	w        io.WriteCloser
}

func (w *documentWriter) SetType(mimetype string) {
	w.document.Type = mimetype
}

func (w *documentWriter) Metadata() models.Metadata {
	if w.document.Metadata == nil {
		w.document.Metadata = make(models.Metadata)
	}

	return w.document.Metadata
}

func (w *documentWriter) CloseMetadata() error {
	return nil
}

func (w *documentWriter) Write(p []byte) (int, error) {
	return w.w.Write(p)
}

func (w *documentWriter) RenderErrors(errors ...models.RenderError) {
	w.document.RenderErrors = append(w.document.RenderErrors, errors...)
}

func (w *documentWriter) Close() error {
	return w.w.Close()
}

func NewDocumentWriter(document *models.Document) DocumentWriter {
	return &documentWriter{
		document: document,
		w:        document.BytesWriter(),
	}
}
