package handlers_test

import (
	"encoding/base64"
	"net/http"
	"testing"

	"github.com/Masterminds/squirrel"
	"github.com/stretchr/testify/assert"

	"orus.io/orus-io/go-orusapi/database"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/testutils"
	"orus.io/orus-io/rednerd/testutils/apitester"
)

func TestAccountTemplateUpdate(t *testing.T) {
	t.Parallel()
	log := testutils.GetLogger(t)
	ctx, cancel := testutils.GetContext(t)
	ctx = log.WithContext(ctx)

	defer cancel()

	tester := apitester.NewAPITester(ctx, t, nil)
	tmpls := []models.Template{
		{
			Account:    "janedoe",
			Name:       "1",
			Language:   "mustache",
			Produces:   "text/plain",
			BodyFormat: "text",
			Body:       "Hello {{name}}!",
		},
		{
			Account:    "janedoe",
			Name:       "2",
			Language:   "mustache",
			Produces:   "text/plain",
			BodyFormat: "text",
			Body:       "Hello {{name}}!",
		},
	}

	t.Cleanup(func() {
		tester.Close()
		tester.Logout()
	})
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Logout()
	tester.Login("janedoe", "janedoe")

	t.Run("Update non-existent template", func(t *testing.T) {
		t.Parallel()
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/unknown").
			JSON(&models.Template{}).
			Expect(t).
			Status(http.StatusNotFound).
			End()

		var httpErr models.Error

		r.JSON(&httpErr)
		assert.Equal(t, "No such template", httpErr.Message)
	})

	t.Run("Update body", func(t *testing.T) {
		t.Parallel()

		t.Cleanup(func() {
			_, _ = database.Exec(tester.DB, squirrel.Delete(models.TemplateDBTable), &log)
		}())

		var responseTemplate models.Template

		tester.CreateTemplate(
			"janedoe",
			&tmpls[0],
			&responseTemplate,
		)

		responseTemplate.Body = "Hi {{name}}!"
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/" + tmpls[0].Name).
			JSON(&responseTemplate).
			Expect(t).
			Status(http.StatusOK).
			End()
		r.JSON(&responseTemplate)
		assert.Equal(t,
			base64.StdEncoding.EncodeToString([]byte("Hi {{name}}!")), responseTemplate.Body)
	})

	t.Run("Rename", func(t *testing.T) {
		t.Parallel()

		t.Cleanup(func() {
			_, _ = database.Exec(tester.DB, squirrel.Delete(models.TemplateDBTable), &log)
		}())

		var responseTemplate models.Template

		tester.CreateTemplate(
			"janedoe",
			&tmpls[1],
			&responseTemplate,
		)

		responseTemplate.Name = "hi"
		r := tester.APITest("template_update").
			Put("/api/v1/template/janedoe/" + tmpls[1].Name).
			JSON(&responseTemplate).
			Expect(t).
			Status(http.StatusOK).
			End()
		r.JSON(&responseTemplate)
		assert.Equal(t, "hi", responseTemplate.Name)
	})
}
